--[[
path: /PetFeeder/
filename: PetFeeder.lua
author: Jeff Parker <jeff3parker@gmail.com>
created: Tue, 22 Jan 2005 14:15:00 -0800
updated: Tue, 22 Jan 2005 21:39:00 -0800

Pet Feeder: a GUI interface allowing you configure happiness level for your pet & 
			drag/drop foods you wish your pet to eat.  When the pet happiness drops below
			the selected threshold will automatically feed your pet.
			To remove a food from the list, simply click on it.
]]


PETFEEDER_ITEM_HEIGHT = 32;
PETFEEDER_ITEMS_SHOWN = 7;

-- Configuration
PF_Config = {
	Enabled = 1,
	Alert = 1,
	Level = 2,
	SortOption = 3,
	AutoFindFood = 0
};

UIPanelWindows["PetFeederFrame"] = { area = "left", pushable = 3 };
PETFEEDER_TAB_SUBFRAMES = { "PetFeeder_OptionsFrame", "PetFeeder_FoodsFrame", "PetFeeder_UnlikedFoodsFrame" };

-- Foods Lists
PetFeeder_Foods = { };
PetFeeder_BadFoods = { };
PetFeeder_PetName = "";
PetFeeder_Pets = { };

-- Variables
PetFeeder_Var = { };

PetFeeder_Var.PetInCombat = false;
PetFeeder_Var.PlayerInCombat = false;
PetFeeder_Var.Searching = false;
PetFeeder_Var.isSitting = false;
PetFeeder_Var.sitPosX = -1;
PetFeeder_Var.sitPosY = -1;
PetFeeder_Var.feedStartTime = 0;
PetFeeder_Var.debug = false;

-- Hooked function variables
Pre_SitOrStand = nil;
Pre_DoEmote = nil;
Pre_Jump = nil;

--[[
=============================================================================
Debug message output 
=============================================================================
]]
function PFDebugMessage(x,y,z) 
	if ( z == "error" ) then 
		DEFAULT_CHAT_FRAME:AddMessage(format("|cffff0000[%s]: %s|r", x, y)) 
	end
	
	if ( PetFeeder_Var.debug ) then
		DEFAULT_CHAT_FRAME:AddMessage(format("|ccfff0000[%s]: %s|r", x, y)) 
	end
	--if ( z == "debug" ) then 
	--	DEFAULT_CHAT_FRAME:AddMessage(format("|cffff0000[%s]: %s|r", x, y)) 
	--end
	--if ( z == "foods" ) then 
	--	DEFAULT_CHAT_FRAME:AddMessage(format("|cffff0000[%s]: %s|r", x, y)) 
	--end
end


--[[
=============================================================================
The loading frame handles getting our AddOn initialized properly 
=============================================================================
]]
function PetFeederLoadingFrame_OnUpdate()
	local curTime = GetTime();
	if ( PetFeederFrame.unregistered ) then
		if ( PetFeederLoadingFrame.lastCheck + 10 <= curTime ) then
			if ( PetFeeder_CheckHappiness() ) then
				PFDebugMessage("PF", "Attempt to feed", "debug");
				PetFeeder_Feed();
			end
			PetFeederLoadingFrame.lastCheck = curTime;
		end
	elseif ( PetFeederFrame.loaded ) then
		PetFeederLoadingFrame.lastCheck = 0;
		PetFeederFrame.unregistered = true;
		PetFeederLoadingFrame:UnregisterEvent("VARIABLES_LOADED");
		PetFeederLoadingFrame:UnregisterEvent("UNIT_NAME_UPDATE");
		--PetFeederLoadingFrame:Hide();
		--PetFeederLoadingFrame = nil;
	elseif ( PetFeederLoadingFrame.loadTime + 10 <= curTime ) then
		PetFeederFrame_LoadData();
		PetFeederLoadingFrame.loadTime = GetTime();
	end
end

function PetFeederLoadingFrame_OnLoad()
	PetFeederLoadingFrame:RegisterEvent("VARIABLES_LOADED");
	PetFeederLoadingFrame:RegisterEvent("UNIT_NAME_UPDATE");
	PetFeederLoadingFrame.init = 0;
	PetFeederLoadingFrame.loadTime = GetTime();

end

function PetFeederLoadingFrame_OnEvent(event, arg1)
	if ( not PetFeederLoadingFrame ) then
		return;
	end
	if ( event == "VARIABLES_LOADED" or event == "UNIT_NAME_UPDATE" ) then
		PetFeederLoadingFrame.init = PetFeederLoadingFrame.init + 1;
	end

	if ( PetFeederLoadingFrame.init >= 2 ) then
		PetFeederFrame_LoadData();
	end

end

--[[
=============================================================================
Initialize data, perform hooks, register for events
=============================================================================
]]
function PetFeederFrame_LoadData()

	if ( UnitClass("player")~="Shaman") then
		return;
	end

	PetFeeder_PetName = UnitName( "pet" );

	-- Check for old version of foods
	if ( PetFeeder_Foods[PetFeeder_PetName] and getn(PetFeeder_Foods[PetFeeder_PetName]) > 0) then
		anItem = PetFeeder_Foods[PetFeeder_PetName][1];
		if ( not anItem.quality ) then
			FixOldInventoryData();
		end
	end
	
	PetFeederFrame.loaded = true;
end

--[[
=============================================================================
Initialize saves, slash cmd and panels
=============================================================================
]]
function PetFeederFrame_OnLoad()
	
	if ( UnitClass("player")~= PETFEEDER_HUNTER ) then
		return;
	end

	-- Register for Events
	this:RegisterEvent("PET_ATTACK_START");
	this:RegisterEvent("PET_ATTACK_STOP");
	this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_PET_CHANGED");
	this:RegisterEvent("PLAYER_ALIVE");
	
	-- Hook functions
	if ( not Pre_DoEmote ) then
		Pre_DoEmote = DoEmote;
		DoEmote = PetFeeder_DoEmote;
	end
	
	if ( not Pre_Jump ) then
		Pre_Jump = Jump;	
		Jump = PetFeeder_Jump;
	end

	local chatParseInfo = { AddOn = "PetFeeder" };
	chatParseInfo.event    = "CHAT_MSG_SPELL_FAILED_LOCALPLAYER";
	chatParseInfo.func     = function(t) PetFeeder_RemoveBadFood(t.food); end;
	chatParseInfo.template = PETFEEDER_FAILED_TO_FEED;
	chatParseInfo.english  = "You fail to perform Feed Pet: Your pet doesn't like that food.";
	chatParseInfo.fields   = {  };

	if ( not PFChatParse_RegisterEvent ) then
		PFDebugMessage("PF", "function PFChatParse_RegisterEvent not defined!", "error");
	end

	PFChatParse_RegisterEvent(chatParseInfo);
	
	chatParseInfo.func     = function(t) PetFeeder_RemoveBadFood(t.food); end;
	chatParseInfo.template = PETFEEDER_FOODTOOLOW;
	chatParseInfo.english  = "You fail to perform Feed Pet: That food's level is not high enough for your pet.";
	PFChatParse_RegisterEvent(chatParseInfo);

	chatParseInfo.event    = "CHAT_MSG_SYSTEM";
	chatParseInfo.func     = function() PetFeeder_DoEmote("SIT"); end;
	chatParseInfo.template = PETFEEDER_AFK;
	chatParseInfo.english  = "You are now AFK: Away from Keyboard";
	chatParseInfo.fields   = {  };
	PFChatParse_RegisterEvent(chatParseInfo);
	
	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("Jeff's "..PETFEEDER_TITLE.." AddOn loaded.  Use /pf");
	end
	UIErrorsFrame:AddMessage(loadMessage, 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);

	RegisterForSave( "PetFeeder_Foods" );
	RegisterForSave( "PetFeeder_BadFoods" );
	RegisterForSave( "PetFeeder_Pets" );
	RegisterForSave("PF_Config");

	-- Register Slash Commands
	SLASH_PetFeeder1 = "/PetFeeder";
	SLASH_PetFeeder2 = "/pf";
	SlashCmdList["PetFeeder"] = function(msg)
		PetFeeder_SlashCmd(msg);
	end

	-- Tab Handling code
	PanelTemplates_SetNumTabs(PetFeederFrame, 3);
	PanelTemplates_SetTab(PetFeederFrame, 1);
	--PanelTemplates_DisableTab(this, 3);
	
	this.loaded = nil;
	
end

--[[
=============================================================================
Called when PF opens or closes
=============================================================================
]]
function PetFeederFrame_OnShow()
	PlaySound("igCharacterInfoOpen");
end

function PetFeederFrame_OnHide()
	PlaySound("igCharacterInfoClose");
end

--[[
=============================================================================
Hooked Functions
 replaces the original calls with our own and then calls the original.  These
 methods are hooked so we can detect when a player performs a specific action
 such as sitting, jumping, etc.
=============================================================================
]]
function PetFeeder_DoEmote(token,...)	

	if ( token == "SIT" ) then
		 PetFeeder_Var.sitPosX, PetFeeder_Var.sitPosY = GetPlayerMapPosition("player");
		 PetFeeder_Var.isSitting = true;
	end
	if ( token == "STAND" ) then
		PetFeeder_Var.isSitting = false;
	end
	
	if ( table.getn(arg) > 0 ) then
		Pre_DoEmote(token,arg[1]);	
	else
		Pre_DoEmote(token);
	end
	
end
function PetFeeder_Jump()
	PetFeeder_Var.isSitting = false;
	Pre_Jump();
end

--[[
=============================================================================
Called when we want to display a list of foods
=============================================================================
]]
local function PetFeeder_BuildDisplayIndices()
	local iNew = 1;
	local index;
	local value;
	
	DisplayIndices = { };
	for index, value in PetFeeder_Foods[PetFeeder_PetName] do
			local displayValue = value.name.." ("..value.quantity..")";			
			DisplayIndices[iNew] = displayValue;
			iNew = iNew + 1;
	end
	DisplayIndices.numEntries = iNew-1;
end
--[[
=============================================================================
Walked through our list of foods and obtains the ItemCount for each
=============================================================================
]]
function updateQuantities()
	for index, value in PetFeeder_Foods[PetFeeder_PetName] do
			-- update the item count
			value.quantity = PetFeeder_GetItemCount( value.name );
	end
end

function PetFeeder_Update()
	local iItem;
			
	updateQuantities();

	PetFeeder_BuildDisplayIndices();
	if( not DisplayIndices ) then
		PetFeeder_BuildDisplayIndices();
	end

	local numEntries = DisplayIndices.numEntries;

	--FauxScrollFrame_Update(PetFeederListScrollFrame, numEntries, PETFEEDER_ITEMS_SHOWN, PETFEEDER_ITEM_HEIGHT);
	FauxScrollFrame_Update(PetFeederListScrollFrame, numEntries, PETFEEDER_ITEMS_SHOWN, nil, nil, nil, nil, nil, nil, PETFEEDER_ITEM_HEIGHT);

	local scrollFrameOffset = FauxScrollFrame_GetOffset(PetFeederListScrollFrame);
	
	for iItem = 1, PETFEEDER_ITEMS_SHOWN, 1 do
		
		local itemIndex = iItem + scrollFrameOffset;
		local buttonItem = getglobal("PetFeederItem"..iItem);
		if ( itemIndex > numEntries ) then
		  buttonItem:Hide();
		else
 			local name = DisplayIndices[itemIndex];
			buttonItem:SetText(name);
			buttonItem:Show();
		end
	end	
end

function togglePetFeeder(tab)

	if ( not tab ) then
		if ( PetFeederFrame:IsVisible() ) then
			HideUIPanel(PetFeederFrame);
		else
			ShowUIPanel(PetFeederFrame);
			local selectedFrame = getglobal(PETFEEDER_TAB_SUBFRAMES[PetFeederFrame.selectedTab]);
			if ( not selectedFrame:IsVisible() ) then
				selectedFrame:Show();
			end
		end
	else
		local subFrame = getglobal(tab);
		if ( subFrame ) then
			PanelTemplates_SetTab(PetFeederFrame, subFrame:GetID() );
			if ( PetFeederFrame:IsVisible() ) then
				if ( subFrame:IsVisible() ) then
					HideUIPanel( PetFeederFrame );
				else
					PlaySound("igCharacterInfoTab");
					PetFeederFrame_ShowSubFrame(tab);
				end
			else
				ShowUIPanel( PetFeederFrame );
				PetFeederFrame_ShowSubFrame(tab);
			end
		end
	end
	
end

function PetFeederFrame_ShowSubFrame(frameName)
	for index, value in PETFEEDER_TAB_SUBFRAMES do
		if ( value == frameName ) then
			getglobal(value):Show();
		else
			getglobal(value):Hide();
		end
	end
end
---------------------------------------------------
--Show config dialog when slash-command is called--
---------------------------------------------------
function PetFeeder_SlashCmd(msg)

	
	if ( not PetFeeder_PetName or PetFeeder_PetName == "" ) then
		UIErrorsFrame:AddMessage(PETFEEDER_ESTABLISH_PET, 0.8, 0, 0, 1.0, UIERRORS_HOLD_TIME);
		return;
	end

	if (msg == "feed" ) then
	  PetFeeder_Feed();
	elseif ( msg == "clear" ) then			
		PetFeeder_ClearFoods();
	elseif ( msg == "clearbad" ) then			
		PetFeeder_ClearBadFoods();
	elseif ( msg == "dump" ) then
		PetFeeder_DebugDump();
	elseif ( msg == "toggledebug" ) then
		PetFeeder_Var.debug = not PetFeeder_Var.debug;
		if ( PetFeeder_Var.debug ) then
			DEFAULT_CHAT_FRAME:AddMessage("debug is ON");
		else
			DEFAULT_CHAT_FRAME:AddMessage("debug is OFF");
		end
	elseif ( msg == "buff" ) then
		PetFeeder_PlayerBuff();
	elseif ( msg == "populate" ) then
		PetFeeder_PopulateFoods();
	else
			togglePetFeeder(nil);
	end
	
end

-- Do not localize this
function PetFeeder_DebugDump()
	local i;
	DEFAULT_CHAT_FRAME:AddMessage("PetFeeder Foods in the order of consumption");
	for i=1, table.getn( PetFeeder_Foods[PetFeeder_PetName] ) do
		DEFAULT_CHAT_FRAME:AddMessage(PetFeeder_Foods[PetFeeder_PetName][i].name);
	end
	DEFAULT_CHAT_FRAME:AddMessage("PetFeeder BadFoods");
	for i=1, table.getn( PetFeeder_BadFoods[PetFeeder_PetName] ) do
		DEFAULT_CHAT_FRAME:AddMessage(PetFeeder_BadFoods[PetFeeder_PetName][i].name);
	end
end

function PetFeeder_GetID(button)
	if ( button == nil ) then
		return 0;
	end

	return (button:GetID())
end

function PetFeederFrame_OnEvent(event, arg1)
	
	if ( PetFeeder_Var.isSitting == true) then
		local posX, posY = GetPlayerMapPosition("player");
		if ( PetFeeder_Var.sitPosX ~= posX ) then
			PetFeeder_Var.isSitting = false;
		elseif (PetFeeder_Var.sitPosY ~= posY ) then
			PetFeeder_Var.isSitting = false;
		end
	end
		
	if ( PetFeederFrame:IsVisible() ) then
		PetFeeder_FoodsFrame_Update();
		PetFeeder_UnlikedFoodsFrame_Update();
	end
	
	if not ( UnitExists("pet") ) then
		PFDebugMessage("PF-Abort", "cannot find a pet", "debug");
		PetFeeder_Var.PetInCombat = false;  -- Pet could have died during combat and we don't get notified
		return;
	end

	PetFeeder_PetName = UnitName( "pet" );
	if ( not PetFeeder_Foods[PetFeeder_PetName] ) then
	  PFDebugMessage("PF", "inserting new pet", "debug");
	  PetFeeder_Foods[PetFeeder_PetName] = {};
	end
	if ( not PetFeeder_BadFoods[PetFeeder_PetName] ) then
	  PFDebugMessage("PF", "inserting new pet", "debug");
	  PetFeeder_BadFoods[PetFeeder_PetName] = {};
	end

	if ( PF_Config.AutoFindFood ) then
		PetFeeder_PopulateFoods();
	end
	
	local eventmsg = "event="..event;
	PFDebugMessage("PF", eventmsg, "debug");
	
	if ( event == "PET_ATTACK_START" ) then
		PetFeeder_Var.PetInCombat = true;
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then		
		PetFeeder_Var.PlayerInCombat = true;
	elseif ( event == "PET_ATTACK_STOP" ) then
		PetFeeder_Var.PetInCombat = false;
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		PetFeeder_Var.PlayerInCombat = false;
	elseif ( event == "PLAYER_PET_CHANGED" ) then
		PetFeeder_FoodsFrame_Update();
		PetFeeder_UnlikedFoodsFrame_Update();
	elseif ( event == "PLAYER_ALIVE" ) then  -- no notification of deaths, be sure to clear these
		PetFeeder_Var.PetInCombat = false;
		PetFeeder_Var.PlayerInCombat = false;
	end

	if ( event == "UNIT_HAPPINESS" and arg1 == "pet" ) then
		-- Check if in Combat
		if ( PetFeeder_Var.PetInCombat or PetFeeder_Var.PlayerInCombat ) then
			PFDebugMessage("PF-Abort", "Can't feed while in combat", "debug");
			return;
		end
		
	-- Check Happiness
	if ( PF_Config.Enabled ) then
		if ( PetFeeder_CheckHappiness() ) then
			PFDebugMessage("PF", "Attempt to feed", "debug");
			PetFeeder_Feed();
		end
	end

	end
end

-- Check player buffs
-- return true if buff enabled we don't want to break by feeding
-- (i.e. FeignDeath or Shadowmeld )
function PetFeeder_PlayerBuff()
	local i = 1;
	local buff;
	local unit = "player";
	buff = UnitBuff(unit,i);
	while buff do
		--PetFeederTooltip:SetUnitBuff( unit, i );
		--DEFAULT_CHAT_FRAME:AddMessage("debug::Player buff::"..i.."="..buff);
		local debuginfo = "Player buff::"..i.."="..buff;
		if ( isMounted( unit, i, buff ) ) then
			PFDebugMessage("PF-Abort", "Player is mounted", "debug");
			return true;
		elseif ( string.find(buff, "Ability_Rogue_FeignDeath") ) then
			PFDebugMessage("PF-Abort", debuginfo, "debug");
			return true;
		elseif ( string.find(buff, "Ability_Ambush") ) then -- shadowmeld
			PFDebugMessage("PF-Abort", debuginfo, "debug");
			return true;
		elseif ( string.find(buff, "DemonBreath") ) then -- water breathing
			PFDebugMessage("PF-Abort", debuginfo, "debug");
			return true;
		end		
		i = i + 1;
		buff = UnitBuff(unit,i);
	end

	-- Check for eating or drinking
	local playerBuffs = {"Interface\\Icons\\INV_Drink_07","Interface\\Icons\\INV_Misc_Fork&Knife"};
	for i=0, 15 do
		buff = GetPlayerBuffTexture(i);
		if ( buff ) then
			for k,v in playerBuffs do
				if ( buff == v ) then
					PFDebugMessage("PF-Abort", buff, "debug");
					return true;
				end
			end
		end
	end
	
	return false;
end

--=============================================================================
-- Return information about the pattern
--
-- pattern	pattern to find
local function BuffInformation( pattern )
	if pattern then
		return string.find( PetFeederTooltipTextLeft2:GetText(), pattern );
	else
		return 1;
	end
end
--=============================================================================

-- Check to see if the player is mounted
function isMounted(unit, i, buff )
	PetFeederTooltip:SetUnitBuff( unit, i );
	if ( string.find(buff, "_Mount_" ) or string.find(buff,"INV_Misc_Foot_Kodo") ) then
		local startpos,endpos,buffValue = BuffInformation(" (%d+)%%");
		if ( buffValue ~= nil ) then
			buffValue = buffValue + 0;
			if ( buffValue >=60 ) then
				return true;
			end
		end
	end
	
	return false;
	
end

-- Check pet debuffs
-- return true if pet has debuff that will break feeding
-- (i.e. poisoned )
function PetFeeder_PetDebuff()
	local i = 1;
	local buff;
	buff = UnitDebuff("pet", i);
	
	while buff do
		local debuginfo = "Pet debuff::"..i.."="..buff;
		--PFDebugMessage("PF", debuginfo, "debugbuff");
		
		if ( string.find(buff, "Spell_Nature_CorrosiveBreath") ) then
			return true;  -- this comes from a Venom Spitter when it poisons the target
		end
		if ( string.find(buff, "Spell_Nature") ) then
			return true;   -- this might be too liberal, but should catch all instances we are searching for
		end
		i = i + 1;
		buff = UnitDebuff("pet", i);
	end
	return false;

end

-- Check Feed Effect
function PetFeeder_HasFeedEffect()
	local i = 1;
	local buff;
	buff = UnitBuff("pet", i);
	
	while buff do
		local debuginfo = "Pet buff::"..i.."="..buff;
		--PFDebugMessage("PF", debuginfo, "debugbuff");

		if ( string.find(buff, "Ability_Hunter_BeastTraining") ) then
			return true;
		end
		i = i + 1;
		buff = UnitBuff("pet", i);
	end
	return false;

end

-- Check Happiness
function PetFeeder_CheckHappiness()

	-- Get Pet Info
	local pet = UnitName("pet");
	local happiness, damage, loyalty = GetPetHappiness();
	
	local level = PF_Config.Level + 1;
	
	-- Check Happiness
	if ( happiness == 0 ) or ( happiness == nil ) then
		PFDebugMessage("PF-Abort", "Unable to determine pet happiness", "debug");
		return false;
	end
	
	-- Check if Feeding is needed

	if ( happiness < level ) then
		PFDebugMessage("PF", "pet isn't happy enough", "debug");

		if not ( PetFeeder_Var.Searching ) then			
			return true;
		else
			PFDebugMessage("PF-Abort", "Pet already searching for food", "debug");
			return false;
		end
	end
	local msg = "Feeding not necessary.  Pet happiness is at threshold:"..PETFEEDER_LEVELS_DROPDOWN[happiness-1].name;
	PFDebugMessage("PF-Abort", msg, "debug");
	return false;
end

-- Feed Pet
function PetFeeder_Feed()

	-- Is there a debuff on the pet that will break the feeding Buff?
	if ( PetFeeder_PetDebuff() ) then
		PFDebugMessage("PF-Abort", "Pet is debuffed", "debug");
		return;
	end

	-- check for player buffs that might interfere with feeding
	-- or result in undesired effects (such as stopping feign death or shadowmelding)
	if ( PetFeeder_PlayerBuff() ) then
		PFDebugMessage("PF-Abort", "Can't feed with these player buffs", "debug");
		return;
	end

	if ( PetFeeder_Var.isSitting ) then
		PFDebugMessage("PF-Abort", "player is sitting", "debug");
		return;
	end
	if ( UnitOnTaxi("player") ) then -- can't feed if flying, boating, etc
		PFDebugMessage("PF-Abort", "player is on a taxi", "debug");
		return;
	end

	if ( CastingBarFrameStatusBar:IsVisible() ) then  -- can't feed if channeling
		PFDebugMessage("PF-Abort", "player is casting", "debug");
		return;
	end

	-- Check if Has Feed Effect
	if ( PetFeeder_HasFeedEffect() ) then
		PFDebugMessage("PF-Abort", "Pet already has Feed effect", "debug");
		return;
	end

	if ( PetFeeder_PlayerBuff() ) then
		PFDebugMessage("PF-Abort", "Can't feed with player buffed like this", "debug");
		return;
	end
	
	updateQuantities();
	PetFeeder_Var.Searching = true;
	if ( PF_Config.Alert ) then
		DEFAULT_CHAT_FRAME:AddMessage(PetFeeder_PetName..PETFEEDER_BEGIN_SEARCH);
	end			

	
	-- 1. Get first available food from our list with quantity > 0
	-- 2. Find the m,n location of it in the pack.
	-- 3. Pick up and eat.
	local lowestQuantity = 99;
	local lowestM = 0;
	local lowestN = 0;
	for index, value in PetFeeder_Foods[PetFeeder_PetName] do
		if ( value.quantity > 0 ) then
		 -- Find lowest instance of the food item
			for m = 0, 4 do
				for n = 1, 18 do
					itemObject = PetFeeder_GetItemObject(m,n);
					if ( itemObject and itemObject.name == value.name ) then
						-- Using this itemCount because we need the actual value in the slot
						local texture, itemCount, locked, quality, readable = GetContainerItemInfo(m,n);
						if ( itemCount < lowestQuantity ) then						
							--DEFAULT_CHAT_FRAME:AddMessage("lowest M = "..m.." lowestN="..n.." lowestqty="..itemCount);
							lowestQuantity = itemCount;
							lowestM = m;
							lowestN = n;
						end
					end
				end
			end
			
			local msg = "Feed item in bag,slot="..lowestM..","..lowestN.." which is "..value.name;
			PFDebugMessage("PF", msg, "debug" );
			
			PickupContainerItem( lowestM, lowestN );

			PetFeeder_Var.feedStartTime = GetTime();
			
			if ( CursorHasItem() ) then				
				DropItemOnUnit("pet");
				PFDebugMessage("PF", "Fed item "..value.name, "debug" );
			end
			if ( CursorHasItem() ) then
				PickupContainerItem(lowestM, lowestN);
			end

			value.quantity = value.quantity - 1;
			PetFeeder_FoodsFrame_Update();
			-- Alert
			if ( PF_Config.Alert ) then
				DEFAULT_CHAT_FRAME:AddMessage(PetFeeder_PetName..PETFEEDER_EATS_A..value.name );
			end

			PetFeeder_Var.Searching = false;
			return;
		 end
	end
	
	-- No Food Could be Found
	PFDebugMessage("PF", "No food could be found to feed the pet", "debug" );
	if ( PF_Config.Alert ) then
		DEFAULT_CHAT_FRAME:AddMessage(PetFeeder_PetName..PETFEEDER_NO_FOOD);
	end
	
	PetFeeder_Var.Searching = false;	
end

------------------
-- Threshold Dropdown
------------------
local function PetFeederFrameDropDown_Initialize()
	local info;
	for i = 1, getn(PETFEEDER_LEVELS_DROPDOWN), 1 do
		info = { };
		info.text = PETFEEDER_LEVELS_DROPDOWN[i].name;
		info.func = PetFeederFrameDropDownButton_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function PetFeederFrameDropDown_OnLoad()
	UIDropDownMenu_Initialize(PetFeederFrameDropDown, PetFeederFrameDropDown_Initialize);
	UIDropDownMenu_SetWidth(80);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", PetFeederFrameDropDown)
end

function PetFeederFrameDropDownButton_OnClick()
	UIDropDownMenu_SetSelectedID(PetFeederFrameDropDown, this:GetID());
	PF_Config.Level = UIDropDownMenu_GetSelectedID(PetFeederFrameDropDown);	
end


-- Sorting Algorithms
function sortFoods()

	updateQuantities();
	
	if ( PF_Config.SortOption == 1 ) then
		return;
	end;
	if ( PF_Config.SortOption == 2 ) then
		sortQuantityHighLow();
	else
		if ( PF_Config.SortOption == 3 ) then
			sortQuantityLowHigh();
		else
			if ( PF_Config.SortOption == 4 ) then
				sortAlphabeticallyHighLow();	
			else
				if ( PF_Config.SortOption == 5 ) then
					sortAlphabeticallyLowHigh();	
				end
			end
		end
	end


end
function sortQuantityHighLow(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.quantity > b.quantity end );
	if ( a and b ) then
		return a.quantity - b.quantity;
	else
		return 0;
	end
end
function sortQuantityLowHigh(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.quantity < b.quantity end );
	if ( a and b ) then
		return b.quantity - a.quantity;
	else
		return 0;
	end
end
function sortQualityHighLow(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.quality > b.quality end );
	if ( a and b ) then
		return a.quality - b.quality;
	else
		return 0;
	end
end
function sortQualityLowHigh(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.quality < b.quality end );
	if ( a and b ) then
		return b.quality - a.quality;
	else
		return 0;
	end
end
function sortAlphabeticallyHighLow(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.name > b.name end );
	if ( a and b ) then
			if ( a.name > b.name ) then
			return 1;
		elseif ( a.name == b.name ) then
			return 0;
		end
		return -1;
	end
	return 0;
end
function sortAlphabeticallyLowHigh(a,b)
	--table.sort( PetFeeder_Foods[PetFeeder_PetName], function(a,b) return a.name < b.name end );
	if ( a and b ) then
		if ( a.name < b.name ) then
			return 1;
		elseif ( a.name == b.name ) then
			return 0;
		end
		return -1;
	end
	return 0;
end


--[[
=============================================================================
The primary sort compare method.  We may call up to to actual sorting methods
depending on the results.  The sorting methods return 1, 0, -1 while
this method results true|false;
returns: 
=============================================================================
]]
local function compareItem(a,b)
	
  local result = 0;

--if ( a ) then
--	DEFAULT_CHAT_FRAME:AddMessage("a.name="..a.name);  
--end
--if ( b ) then
--	DEFAULT_CHAT_FRAME:AddMessage("b.name="..b.name);  
--end
--if ( sortMethod1 ) then
--	DEFAULT_CHAT_FRAME:AddMessage("sortMethod1 is valud");  
--end
--if ( sortMethod2 ) then
--	DEFAULT_CHAT_FRAME:AddMessage("sortMethod2 is valud");  
--end

  if ( sortMethod1 ) then
  	result = sortMethod1(a,b);
  end
  if ( sortMethod2 and result == 0) then
  	result = sortMethod2(a,b);
  end
  

  return result > 0;
  
  --if ( result > 0 ) then
  --	return true;
  --elseif ( result < 0 ) then
  --	return false;
  --end
  
  --return true;

end


--[[
=============================================================================
Sorter
=============================================================================
]]
function sortTheFoods(list)
	if ( PF_Config.SortOption == 1 ) then
		return;
	end	

	table.sort( list, compareItem );
end

-----------------------------
--PetFeeder Checkbuttons--
-----------------------------
function PetFeeder_PF_Enabled_CheckBt_Update(whatValue)
	PF_Config.Enabled = whatValue;
end

function PetFeeder_PF_AutoFindFood_CheckBt_Update(whatValue)
	PF_Config.AutoFindFood = whatValue;
end

function PetFeeder_PF_Alerts_CheckBt_Update(whatValue)
	PF_Config.Alert = whatValue;
end

--[[
=============================================================================
 Food Management Routines 
=============================================================================
]]

--[[
=============================================================================
Adds a food item to the Food list or the Bad food list.  Checks to ensure
that the food isn't already in the table and checks to ensure we aren't
re-adding food the Pet doesn't like.
params:
 - foodList: list to add the item to.
 - value   : table object for the item
=============================================================================
]]
function PetFeeder_AddFood(foodList, value )
	
	-- don't add if already in the table
	for i=1, table.getn( foodList ) do
	  if ( foodList[i].name == value.name ) then
		return true;
	  end
	end

	-- Check to see if this food is liked, we could be adding to the BadFoods table
	-- but that will be handled above...
	for i=1, table.getn( PetFeeder_BadFoods[PetFeeder_PetName] ) do
		if ( PetFeeder_BadFoods[PetFeeder_PetName][i].name == value.name ) then
			return false;
		end
	end

	table.insert(foodList, value );

	sortFoods();
	
	return true;
end

--[[
=============================================================================
Removes a food item from the Food list or the Bad food list.  A quick pattern
match is required to grab the item name from the rendered text.
ie. 'Tuna Snack (21)' would return simply 'Tuna Snack'
params:
 - foodList: the name of the list to remove the item from.
 - itemName: name of the item to remove
=============================================================================
]]
function PetFeeder_RemoveFood(foodList, itemName)
	
	local _,_,name = string.find(itemName, "(.*) %(%d+%)");
	
	if ( not name ) then
		name = itemName;
	end

	if ( name ) then 
		local msg = "Try to remove "..name.." from Food list."..itemName;
		PFDebugMessage("PF", msg, "debug");
	  
	  for i=1, table.getn( foodList ) do

	    if ( foodList[i].name == name ) then
	    	local msg = "Removed "..name.." from Food list.";
	    	PFDebugMessage("PF", msg, "debug");
	        table.remove( foodList, i );
	      return;
	    end
	  end
	end	
end

--[[
=============================================================================
This method gets called by the ChatMsg callback functions when it detects
that a pet doesn't like the food we fed it.
- Bad food will be the first one readily available
- Only gets called when a bad food item is detected
- If the user manually feeds, the timer which is set when WE feed, will be off
  by more than 2 clicks.  We don't want to remove any foods from our list because
  we don't know what the pet didn't like.
  params:
   - t: nil
=============================================================================
]]
function PetFeeder_RemoveBadFood(t)
	
	updateQuantities();
	
	PFDebugMessage("PF", "Bad food detected", "debug");
	if ( (GetTime() - PetFeeder_Var.feedStartTime) >= 2 ) then
		PFDebugMessage("PF", "Not removing bad food as feed wasn't initiated by PF", "debug");
		return;
	end
	for index, value in PetFeeder_Foods[PetFeeder_PetName] do
		if ( value.quantity > 0 ) then			
			if ( PF_Config.Alert ) then
				DEFAULT_CHAT_FRAME:AddMessage(PETFEEDER_REMOVE_FOOD..value.name);
			end
			
			table.remove( PetFeeder_Foods[PetFeeder_PetName], index );
			
			PetFeeder_AddFood( PetFeeder_BadFoods[PetFeeder_PetName], value );
		end
	end	
	
	PetFeeder_UnlikedFoodsFrame_Update();
	PetFeeder_FoodsFrame_Update();
end

--[[
=============================================================================
Gets called when we want to drop an item onto the list.  Identifies which
item in bag,slot is being dropped, gets into about that item and then attempts
to add it to the list.
params:
 - foodList: the name of the list to add the item to.
=============================================================================
]]
function PetFeeder_DroptheItem( foodList, accept )
	
	if CursorHasItem() and PetFeeder_PickedupItem then
		--local itemName,itemCount,itemQuality,texture,linktext,itemColor,itemID = PetFeeder_GetItemInfo(PetFeeder_PickedupItem.bag,PetFeeder_PickedupItem.slot);
		local value = PetFeeder_GetItemObject( PetFeeder_PickedupItem.bag,PetFeeder_PickedupItem.slot );
		if ( value ) then
			if ( isFood( value.texture, value.name ) or accept == "any" ) then

				if ( PetFeeder_AddFood( foodList, value ) == false ) then
					if ( PF_Config.Alert ) then
						DEFAULT_CHAT_FRAME:AddMessage(PETFEEDER_UNLIKED_FOOD..itemName);				
					end
					UIErrorsFrame:AddMessage(PETFEEDER_UNLIKED_FOOD..itemName, 0.8, 0, 0, 1.0, UIERRORS_HOLD_TIME);
				end

			end
		end
		ResetCursor();
		PetFeeder_TakeItemOffCursor(PetFeeder_PickedupItem.bag,PetFeeder_PickedupItem.slot);			
	end  -- if has item and we know where it came from

	PetFeeder_PickedupItem = nil;	

end


--[[
=============================================================================
Returns the item name;
params:
 - bag: the bag number of the item
 - slot: the slot number of the item
=============================================================================
]]
function PetFeeder_GetItemName(bag, slot)
  --local name,totalCount,quality,texture,linktext = PetFeeder_GetItemInfo(bag,slot);   
  local value = PetFeeder_GetItemObject( m, n );
  
  if ( value ) then
  	return value.name;
  end
  
  return nil;
end

--[[
=============================================================================
Looks through all bags/slots to find the item and adds up the total quantity
for it.
params:
 - itemName: name of the item
=============================================================================
]]
function PetFeeder_GetItemCount(itemName)
	local totalItemCount = 0;
	local itemQuality;
	
  -- calc total number of items
  for m = 0, 4 do
	for n = 1, 18 do
      local curLinkText = GetContainerItemLink(m, n);
      local curname;
	  if curLinkText then
	  	_,_,curname = string.find(curLinkText, "^.*%[(.*)%].*$");
	  end
	  if ( curname == itemName ) then
      	texture, itemCount, locked, quality, readable = GetContainerItemInfo(m,n);
      	itemQuality = quality;
      	if ( itemCount ) then
      		totalItemCount = totalItemCount + itemCount;
      	end
      end
    end
  end
  
  return totalItemCount,itemQuality;
  
end

function PetFeeder_GetItemObject(bag,slot)
  
  local value = { };
  local linktext = GetContainerItemLink(bag, slot);
  local itemTexture, itemCount, locked, itemQuality, readable = GetContainerItemInfo(bag,slot);
  local itemColor, itemID, itemName;
  local restores, overTime;
  
  value.quality = 61;

	-- no sense processing if there isn't an item
  if ( not linktext ) then
  	return nil;
  end
  
	for itemColor, itemID, itemName in string.gfind(linktext, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r") do
		if ( itemColor and itemID and itemName ~= "" ) then
			  value.name = itemName;
			  value.color=itemColor;
			  value.id=itemID;
			--DEFAULT_CHAT_FRAME:AddMessage("color="..itemColor.."  itemID="..itemID.."  name="..itemName.." texture="..itemTexture);  	
		end
	end

	-- Look for 'Use: Restores XXXX health over YY seconds'
	-- Regex looks for 'Use:', a space, 1 or more characters, a space, 1 or more digits
	-- Return the digits as our version of quality
	PetFeederTooltip:SetHyperlink( "item:"..value.id );
	if ( PetFeederTooltipTextLeft2:GetText() ) then
		--for restores in string.gfind(PetFeederTooltipTextLeft2:GetText(), PETFEEDER_RESTORES.." (%d+)") do
		for restores in string.gfind(PetFeederTooltipTextLeft2:GetText(), ITEM_SPELL_TRIGGER_ONUSE.." %w+ (%d+)") do		
			if ( restores ~= "" ) then
				  value.quality = restores;
			end
		end
	end
	if ( PetFeederTooltipTextLeft3:GetText() ) then
		--for restores in string.gfind(PetFeederTooltipTextLeft3:GetText(), PETFEEDER_RESTORES.." (%d+)") do
		for restores in string.gfind(PetFeederTooltipTextLeft3:GetText(), ITEM_SPELL_TRIGGER_ONUSE.." %w+ (%d+)") do		
			if ( restores ~= "" ) then
				  value.quality = restores;
			end
		end
	end
  
  value.texture = itemTexture;
  value.link = linktext;


  local totalItemCount = PetFeeder_GetItemCount( name );
  value.quantity = totalItemCount;

--  if ( value.name and value.name == "Mystery Meat" ) then 
--    DEFAULT_CHAT_FRAME:AddMessage("object::color="..value.color.."  itemID="..value.id.."  name="..value.name);
--	  DEFAULT_CHAT_FRAME:AddMessage("object::texture="..value.texture.."  quality="..value.quality.."  link="..value.link);
--	  DEFAULT_CHAT_FRAME:AddMessage("object::quantity="..value.quantity);
--  else
--    DEFAULT_CHAT_FRAME:AddMessage("null name in bag="..bag.." slot="..slot);
--  end
  
  if ( value.name ) then
  	return value;
  end
 
  return nil;
  
end

function PetFeeder_TakeItemOffCursor(srcBag, srcSlot)
	if srcBag == -1 then
		PickupInventoryItem(srcSlot);
	else
		PickupContainerItem(srcBag, srcSlot);
	end
end

--[[
=============================================================================
Algorithm to auto-populate foods into our food list.  Uses pattern matching 
to find foods based on popular words associated with foods in the game.
Alt Algorithm: uses texture to determine whether an item is a food item or not.
=============================================================================
]]
function PetFeeder_PopulateFoods()

  --PetFeeder_ClearFoods();
  -- Walk through each inventory item
  for m = 0, 4 do
	for n = 1, 18 do
      -- local curLinkText = GetContainerItemLink(m, n);
      --local texture, itemCount, locked, itemQuality, readable = GetContainerItemInfo(m,n);
      --local itemName,itemCount,itemQuality,texture,linktext,itemColor,itemID = PetFeeder_GetItemInfo(m,n);
	  --if linktext then
	  	
	  --	PFDebugMessage( "PF-FoodTexture", itemName.." - "..texture.." link="..linktext, "debug" );
  	  --  PFDebugMessage( "PF", itemName, "foods" );
		--if ( isFood( texture, itemName ) ) then 
		--  --local value = { name = itemName, quantity = itemCount, quality = itemQuality, link = curLinkText }
		--  local value = { name = itemName, quantity = itemCount, quality = itemQuality, link = linktext, color=itemColor, id=itemID }
		--  PetFeeder_AddFood( PetFeeder_Foods[PetFeeder_PetName], value );
		--end
	  --end
	  local value = PetFeeder_GetItemObject( m,n );
	  if ( value ) then
		  if ( isFood( value.texture, value.name ) ) then
			PetFeeder_AddFood( PetFeeder_Foods[PetFeeder_PetName], value );
		  end
	  end
	  
	end
  end
  
  PetFeeder_FoodsFrame_Update();
end

--[[
=============================================================================
Determine whether the item is food or not.  We try three things to figure this out
1. If the texture contains the word _Food_ or _Misc_Bowl_ we assume its a food item
2. If the texture contains certain woods (ie: Weapon, Arrow) we eliminate it as a choice
3. Everything else goes through our food filter list to see if they contain
   keywords for food items.
Even if we mis-identify something as a food item, it will get eliminated from
the pet's diet list as soon as it tries to eat it and the Add_Food routine
will eliminate it for us.
=============================================================================
]]
function isFood( texture, itemName )
	
	-- This section looks at the Texture to determine whether its a food
	-- or not.  We do not need to externalize these values as they are codeified values.
	if ( string.find( texture, "_Food_" ) ) then
		return true;
	elseif ( string.find( texture, "_Misc_Bowl_" ) ) then
		return true;
	elseif ( string.find( texture, "_Fish_" ) ) then
		return true;
	end
	
	-- Eliminate some display textures we know won't be foods
	local nonFoodTextures = { "_Scroll_", "_Weapon_", "_Arrow_", "_Rune_", "_Bandage_", "_Letter_", "_Fabric_" };
	local i;
	for  i=1, table.getn( nonFoodTextures ) do
		if ( string.find( texture, nonFoodTextures[i] ) ) then
			return false;
		end
	end

	-- See if itemName matches our food set
	-- List is is externalized and contains various words 
	local i;
	for i=1, table.getn( FOOD_SEARCH ) do		  	
		if ( string.find(itemName, FOOD_SEARCH[i]) ) then
			return true;
		end
	end
	
	return false;
	
end

--=============================================================================
-- Return information about the pattern
--
-- pattern	pattern to find
local function BuffInformation( pattern )
	if pattern then
		return string.find( PetFeederTooltipTextLeft2:GetText(), pattern );
	else
		return 1;
	end
end
--=============================================================================


-- Called when you click on a Tab
function PetFeederTab_OnClick()
	if ( this:GetName() == "PetFeederFrameTab1" ) then
		togglePetFeeder("PetFeeder_OptionsFrame");
	elseif ( this:GetName() == "PetFeederFrameTab2" ) then
		togglePetFeeder("PetFeeder_FoodsFrame");
	elseif ( this:GetName() == "PetFeederFrameTab3" ) then
		togglePetFeeder("PetFeeder_UnlikedFoodsFrame");
	end
	PlaySound("igCharacterInfoTab");
end

--[[
=============================================================================
Hooked function that gets called when a user picks up an item
=============================================================================
]]
local PetFeeder_Save_PickupContainerItem = PickupContainerItem;
PickupContainerItem = function (bag,slot)
	PetFeeder_PickedupItem = { };
	PetFeeder_PickedupItem.bag = bag;
	PetFeeder_PickedupItem.slot = slot;
	
	return PetFeeder_Save_PickupContainerItem(bag,slot);
end

--[[
=============================================================================
Hooked function that gets called when a user picks up an item
=============================================================================
]]
local PetFeeder_Save_PickupInventoryItem = PickupInventoryItem;
PickupInventoryItem = function (slot)
	PetFeeder_PickedupItem = { };
	PetFeeder_PickedupItem.bag = -1;
	PetFeeder_PickedupItem.slot = slot;
	return PetFeeder_Save_PickupInventoryItem(slot);
end

function PetFeederItemButton_OnEnter(list, itemIndex)
	
	local link = nil;
	
	if ( list[itemIndex].id ) then
		link = "item:"..list[itemIndex].id;
	end
	
	if( link ) then
		GameTooltip:SetOwner(this,"ANCHOR_RIGHT");
		GameTooltip:SetHyperlink(link);
		GameTooltip:Show();
	end
end

function PetFeederItemButton_OnLeave()
	if( PetFeederFrame.TooltipButton ) then
		PetFeederFrame.TooltipButton = nil;
		HideUIPanel(PetFeederDisplayTooltip);
	end
end

function FixOldInventoryData()
	local itemObject;
	
	for index, value in PetFeeder_Foods[PetFeeder_PetName] do
		for m = 0, 4 do
			for n = 1, 18 do
				itemObject = PetFeeder_GetItemObject(m,n);
				if ( itemObject and itemObject.name == value.name ) then
					value = itemObject;
					break;
				end
			end
			if ( itemObject and itemObject.name == value.name ) then
				break;
			end
		end		
	end
end