--[[
path: /PetFeeder/
filename: PetFeeder_FoodsFrame.lua
author: Jeff Parker <jeff3parker@gmail.com>
created: Tue, 22 Jan 2005 14:15:00 -0800
updated: Tue, 22 Jan 2005 21:39:00 -0800

Pet Feeder: a GUI interface allowing you configure happiness level for your pet & 
			drag/drop foods you wish your pet to eat.  When the pet happiness drops below
			the selected threshold will automatically feed your pet.
			To remove a food from the list, simply click on it.
]]

--[[
=============================================================================
Called when we want to display a list of foods
=============================================================================
]]
function PetFeeder_FoodsFrame_OnReceiveDrag()
	PetFeeder_DroptheItem( PetFeeder_Foods[PetFeeder_PetName], "foods" );	
	PetFeeder_FoodsFrame_Update();
end

function PetFeeder_FoodsFrame_OnShow()
	PetFeeder_FoodsFrame_Update();
	if ( PF_Config.SortOption ) then
		UIDropDownMenu_SetSelectedID(PetFeeder_FoodsFrameSortOptionDropDown, PF_Config.SortOption, PETFEEDER_SORTOPTION_DROPDOWN);	
		UIDropDownMenu_SetText(PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption].name, PetFeeder_FoodsFrameSortOptionDropDown);
		sortMethod1 = PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption].func;
	end
	if ( PF_Config.SortOption2 ) then
		UIDropDownMenu_SetSelectedID(PetFeeder_FoodsFrameSortOption2DropDown, PF_Config.SortOption2, PETFEEDER_SORTOPTION_DROPDOWN);	
		UIDropDownMenu_SetText(PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption2].name, PetFeeder_FoodsFrameSortOption2DropDown);
		sortMethod2 = PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption2].func;
	end
end

--[[
=============================================================================
Called when we want to display a list of foods
=============================================================================
]]
function PetFeeder_FoodsFrame_Update()
	local iItem;

	--updateQuantities();
	sortTheFoods(PetFeeder_Foods[PetFeeder_PetName]);

	local numEntries = table.getn(PetFeeder_Foods[PetFeeder_PetName] );

	--FauxScrollFrame_Update(PetFeeder_FoodsFrameListScrollFrame, numEntries, PETFEEDER_ITEMS_SHOWN, nil, nil, nil, nil, nil, nil, PETFEEDER_ITEM_HEIGHT);
	FauxScrollFrame_Update(PetFeeder_FoodsFrameListScrollFrame, numEntries, PETFEEDER_ITEMS_SHOWN, PETFEEDER_ITEM_HEIGHT, nil, nil, nil, nil, nil, PETFEEDER_ITEM_HEIGHT);
	
	local scrollFrameOffset = FauxScrollFrame_GetOffset(PetFeeder_FoodsFrameListScrollFrame);

	for iItem = 1, PETFEEDER_ITEMS_SHOWN, 1 do
		
		local itemIndex = iItem + scrollFrameOffset;
		local buttonItem = getglobal("PetFeeder_FoodsFrameItem"..iItem);
		if ( itemIndex > numEntries ) then
		  buttonItem:Hide();
		else
			iconTexture = getglobal("PetFeeder_FoodsFrameItem"..iItem.."ItemIconTexture");
			local value = PetFeeder_Foods[PetFeeder_PetName][itemIndex];
			
			if ( value.texture ) then
				iconTexture:SetTexture( value.texture );
			end
 			local name = value.name.." ("..value.quantity..")";
 			if ( value.quality ) then
 				name = name.." - quality: "..value.quality;
 			end
 			--if ( value.itemLink ) then
 			--	name = value.itemLink;
 			--end

			buttonItem:SetText(name);
			buttonItem:Show();
		end
	end	
end

--[[
=============================================================================
Called when Clear Foods button is clicked
=============================================================================
]]
function PetFeeder_FoodsFrameClearFoodsButton_Update()
	PetFeeder_ClearFoods();
end

function PetFeeder_ClearFoods()
	PetFeeder_PetName = UnitName( "pet" );
	PetFeeder_Foods[PetFeeder_PetName] = {};
	PetFeeder_FoodsFrame_Update();
end

--[[
=============================================================================
Called when Clear Foods button is clicked
=============================================================================
]]
function PetFeeder_FoodsFrameUpdateFoodsButton_Update()
	PetFeeder_PopulateFoods();
end

--[[
=============================================================================
Called when the button item is clicked
=============================================================================
]]
function PetFeeder_FoodsFrameItemButton_OnClick(button,name)
	if ( button == "LeftButton" ) then
		PetFeeder_RemoveFood( PetFeeder_Foods[PetFeeder_PetName], name );
		PetFeeder_FoodsFrame_Update();
	end
end


------------------
-- Sort Option Dropdown
------------------
local function PetFeeder_FoodsFrameSortOptionDropDown_Initialize()
	local info;
	for i = 1, getn(PETFEEDER_SORTOPTION_DROPDOWN), 1 do
		info = { };
		info.text = PETFEEDER_SORTOPTION_DROPDOWN[i].name;
		info.func = PetFeeder_FoodsFrameSortOptionDropDownButton_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end
local function PetFeeder_FoodsFrameSortOption2DropDown_Initialize()
	local info;
	for i = 1, getn(PETFEEDER_SORTOPTION_DROPDOWN), 1 do
		info = { };
		info.text = PETFEEDER_SORTOPTION_DROPDOWN[i].name;
		info.func = PetFeeder_FoodsFrameSortOption2DropDownButton_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function PetFeeder_FoodsFrameSortOptionDropDown_OnLoad()
	UIDropDownMenu_Initialize(PetFeeder_FoodsFrameSortOptionDropDown, PetFeeder_FoodsFrameSortOptionDropDown_Initialize);
	UIDropDownMenu_SetWidth(150);
	UIDropDownMenu_SetButtonWidth(48);
	UIDropDownMenu_JustifyText("LEFT", PetFeeder_FoodsFrameSortOptionDropDown)
end

function PetFeeder_FoodsFrameSortOptionDropDownButton_OnClick()
	UIDropDownMenu_SetSelectedID(PetFeeder_FoodsFrameSortOptionDropDown, this:GetID());
	PF_Config.SortOption = UIDropDownMenu_GetSelectedID(PetFeeder_FoodsFrameSortOptionDropDown);
	sortMethod1 = PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption].func;
	sortTheFoods(PetFeeder_Foods[PetFeeder_PetName]);
	PetFeeder_FoodsFrame_Update();
end

function PetFeeder_FoodsFrameSortOption2DropDown_OnLoad()
	UIDropDownMenu_Initialize(PetFeeder_FoodsFrameSortOption2DropDown, PetFeeder_FoodsFrameSortOption2DropDown_Initialize);
	UIDropDownMenu_SetWidth(150);
	UIDropDownMenu_SetButtonWidth(48);
	UIDropDownMenu_JustifyText("LEFT", PetFeeder_FoodsFrameSortOption2DropDown)
end

function PetFeeder_FoodsFrameSortOption2DropDownButton_OnClick()
	UIDropDownMenu_SetSelectedID(PetFeeder_FoodsFrameSortOption2DropDown, this:GetID());
	PF_Config.SortOption2 = UIDropDownMenu_GetSelectedID(PetFeeder_FoodsFrameSortOption2DropDown);
	sortMethod2 = PETFEEDER_SORTOPTION_DROPDOWN[PF_Config.SortOption2].func;
	sortTheFoods(PetFeeder_Foods[PetFeeder_PetName]);
	PetFeeder_FoodsFrame_Update();
end
